%%%=========================================================
%%% OBJECTIVE: Compare the management performance between Bio and ES
%%% strategy, in terms of population and ES values
%%%========================================================


%%-----------------------------%%
%%  expected nb groups compare %%
%%-----------------------------%%
%%% ES strategy
load P_lossrate_avg.mat
M_ES=zeros(S,S);  % define transition probability matrix for Markov Chain

for j=1:length(policynew_ES) % j is the nb of realistic optimal actions (policynew dim=78 including repeated actions, the total combinations of action possible is 68)
    optimalAction=policynew_ES(j);
    for i=1:S     
    M_ES(i,:)=P{optimalAction}(i,:); % each row i representing a distribution from state i to all possible states under the optimal action
    end
end

% the expected nb of species extant with time under ES strategy
T=nb_node+1;  % change T to see the trend
nbSurvival_ES=zeros(1,T);
P0=zeros(1,S); % define a vector of S representing which state is the initial state (=1 otherwise=0)
id=getState_back(ones(1,nb_node))+1; % the state where all groups are present belongs to which state number?
P0(1,id)=1; % initial state is S, so S=1

for t=1:T-1
    Pt=P0*M_ES^t; % Pt is a vector of S representing the probability of having each of S states
    for i= 1:S
        %popback is the population corresponding to each flyway state
        pop=popback(i)*Pt(i);
        
        %State=getState(i-1,J);
        %nb=sum(State)*Pt(i);
        if (i==1)
            popnew=pop;
        else
            popnew=popnew+pop;
        end
    end
    if (t==1 && i==S)
        nbSurvival_ES(1,t)=breedingpop;
    end
    nbSurvival_ES(1,t+1)=popnew;    
end

%%% Bio strategy
M_Bio=zeros(S,S);  % define transition probability matrix for Markov Chain

for j=1:length(policynew_Bio) % j is the nb of realistic optimal actions (policynew dim=78 including repeated actions, the total combinations of action possible is 68)
    optimalAction=policynew_Bio(j);
    for i=1:S     
    M_Bio(i,:)=P{optimalAction}(i,:); % each row i representing a distribution from state i to all possible states under the optimal action
    end
end

% the expected nb of species extant with time under ES strategy
T=nb_node+1;  % change T to see the trend
nbSurvival_Bio=zeros(1,T);
P0=zeros(1,S); % define a vector of S representing which state is the initial state (=1 otherwise=0)
id=getState_back(ones(1,nb_node))+1; % the state where all groups are present belongs to which state number?
P0(1,id)=1; % initial state is S, so S=1

for t=1:T-1
    Pt=P0*M_Bio^t; % Pt is a vector of S representing the probability of having each of S states
    for i= 1:S
        %popback is the population corresponding to each flyway state
        pop=popback(i)*Pt(i);
        
        %State=getState(i-1,J);
        %nb=sum(State)*Pt(i);
        if (i==1)
            popnew=pop;
        else
            popnew=popnew+pop;
        end
    end
    if (t==1 && i==S)
        nbSurvival_Bio(1,t)=breedingpop;
    end
    nbSurvival_Bio(1,t+1)=popnew;     
end

dif=nbSurvival_ES-nbSurvival_Bio;
figure()
subplot(4,1,1:3)
plot(nbSurvival_ES,'.-')
hold on
plot(nbSurvival_Bio,'.-')
hold off
ylabel('Population')
legend('ES strategy','Bio strategy','Location','Northeast')
legend boxoff
subplot(4,1,4)
plot(dif,'m.-')
xlabel('Time step')
ylabel('Population difference')
legend('Population loss from ES strategy compare to Bio strategy ')
legend boxoff

% how much % of loss in terms of Bio when we move from Bio to ES strategy
sum1=abs(sum(dif));
sumbio=sum(nbSurvival_Bio);
sum1/sumbio % %7.97

%%%----------------------%%
%%%  expected ES reward  %%
%%%----------------------%%
% the expected ES reward with time under ES strategy
ESreward_ES=zeros(1,T);
P0=zeros(1,S); % define a vector of S representing which state is the initial state (=1 otherwise=0)
id=getState_back(ones(1,nb_node))+1; % the state where all groups are present belongs to which state number?
P0(1,id)=1; % initial state is S, so S=1

for t=1:T-1
    Pt=P0*M_ES^t; % Pt is a vector of S representing the probability of having each of S states
    for i= 1:S % among 256 states, only 78 states got realistic actions
        reward=R_ES(i,policy_ES(i))*Pt(i);
        if (i==1)
            rewardnew=reward;
        else
            rewardnew=rewardnew+reward;
        end
    
    if (t==1 && i==S)
        ESreward_ES(1,t)=sum(ES);
    end
    end
        ESreward_ES(1,t+1)=rewardnew;
end

% the expected ES reward with time under Bio strategy
ESreward_Bio=zeros(1,T);
P0=zeros(1,S); % define a vector of S representing which state is the initial state (=1 otherwise=0)
id=getState_back(ones(1,nb_node))+1; % the state where all groups are present belongs to which state number?
P0(1,id)=1; % initial state is S, so S=1

for t=1:T-1
    Pt=P0*M_Bio^t; % Pt is a vector of S representing the probability of having each of S states
    for i= 1:length(policynew_Bio) % among 256 states, only 78 states got realistic actions
        reward=R_ES(ttt2(i),policynew_Bio(i))*Pt(ttt2(i));
        if (i==1)
            rewardnew=reward;
        else
            rewardnew=rewardnew+reward;
        end
    
    if (t==1 && i==length(policynew_Bio))
        ESreward_Bio(1,t)=sum(ES);
    end
    end
        ESreward_Bio(1,t+1)=rewardnew;
end

dif=ESreward_ES-ESreward_Bio;
figure()
subplot(4,1,1:3)
plot(ESreward_ES,'.-')
hold on
plot(ESreward_Bio,'.-')
hold off
ylabel('ES values')
legend('ES strategy','Bio strategy','Location','Northeast')
legend boxoff
subplot(4,1,4)
plot(dif,'m.-')
xlabel('Time step')
ylabel('ES value difference')
legend('ES value gained under ES strategy compare to Bio strategy ')
legend boxoff

% how much ES additional reached under ES strategy
sum1=sum(dif);
sumes=sum(ESreward_Bio);
sum1/sumes % (ESreward_ES-ESreward_Bio)/ESreward_ES. ES strategy could reach an additional 38.45% of ES compare to Bio strategy
